#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# Simple.py
#
# DESCRIPTION:
# Implements a basic form template
#
# NOTES:
# While functional, the primary purpose of this wizard is
# as a "learning-by-example" tool.


from gnue.designer.forms.TemplateSupport import *
import string

# NOTE: It is VERY important that in any references to a
#    "<insert type here> wizard", the word "wizard" must be
#    in lower case, as many proper names such as "Foobar Wizard"
#    have been trademarked. (No, I'm not kidding :)
#
# TODO: Make sure "Foobar Wizard" is not trademarked.
# TODO: If not, should we?
#


class SimpleFormTemplate(FormTemplate):


  # The first step in our wizard.
  # The template parser will initially
  # call GetStep(FIRST_STEP).
  FIRST_STEP = '0'


  ###############
  #
  # Initialize any runtime variables
  #
  def Start(self, form, current):
    self.form = form



  ###############
  #
  # Return the markup for a specific page
  #
  def GetStep(self, stepCode):

    #
    # Step #1 / Get Title, et al
    #
    if stepCode == '0':
      return   { 'title': 'Basic Form Information',
                 'content': (WizardText('Welcome to the sample form wizard.'),
                             WizardText('To create your form, I need to know some basic information.\n\n'
                                        'First, what shall I call your form? This name will appear in '
                                        'the title bar.'),
                             WizardInput('title', label='Form Title:', required=1,
                                         size=40),
                             WizardText('What connection should this form use to connect to the database?'),
                             WizardInput('connection',label='Connection:', required=1,
                                         set=self.GetAvailableConnections()),
                             WizardText('You may be asked to login to this connection.'),
                            ),
                 'prev': None,
                 'next': '1' }


    #
    # Step #2 / Get Base Table
    #
    elif stepCode == '1':

      # TODO: If the connection changed between steps,
      # TODO: variables['table'] and variables['fields']
      # TODO: should be cleared.

      return   { 'title': 'Select Base Table/Source',
                 'content': (WizardText('Now, please select the base table for your form.'),
                             WizardInput('table', label='Base Table:', required=1, lines=5,
                                         set=self.GetAvailableSources(self.variables['connection'])), ),
                 'prev': '0',
                 'next': '2' }


    #
    # Step #3 / Get Columns to Include
    #
    elif stepCode == '2':

      # TODO: If the table changed between steps,
      # TODO: variables['fields'] should be cleared.

      return   { 'title': 'Select Fields to Include',
                 'content': (WizardText('Which fields shall I include in your form?'),
                             WizardInput('fields', label='Columns:', required=1,
                                         maxSelections=-1, orderable=1,
                                         set=self.GetAvailableFields( \
                                                self.variables['connection'],
                                                self.variables['table'])),
                             WizardInput('arrangement', label='Arrangement Method:',
                                         required=1,
                                         set=(('left','Single Record with Labels to Left'),
                                              ('above','Single Record with Labels Above'),
                                              ('grid','Grid Format')))),
                 'prev': '1',
                 'next': None }



  ###############
  #
  # Verify contents of current step
  # Return None if no problems, otherwise
  # return a tuple of error message strings
  #
  def ValidateStep(self, stepCode):

    # The Simple wizard uses basic "required"
    # settings in the page markup, so doesn't
    # need any special validation.
    return None



  ###############
  #
  # We have all the data, so generate our form. This
  # is called after the user clicks the "Finish" button.
  # No more user input is allowed at this point.
  #
  def Finalize(self):

    # We will use the table name as the basis for all our
    # object names. We will add a prefix based on the object
    # type to the table name. Capitalize the first letter and
    # strip all spaces...
    if len(self.variables['table']) == 1:
      tableKey = string.upper(self.variables['table'])
    else:
      tableKey = string.upper(self.variables['table'][0]) + \
                    string.replace(self.variables['table'][1:],' ','_')

    # Is this a grid style form?
    multirecord = self.variables['arrangement'] == 'grid'

    # Will labels appear above or to the left of the entries?
    # Note: this results in false for both "grid" and "above"
    leftlabels = self.variables['arrangement'] == 'left'

    # Set the basic attributes of the form
    self.ModifyElement(self.form, title=self.variables['title'])

    # Create a single datasource based on user's input
    datasource = self.AddElement('datasource', self.form,
       {  'database': self.variables['connection'],
          'table': self.variables['table'],
          'type': 'object',
          'name': 'dts%s' %  tableKey })

    page = None
    for child in self.form._children:
      if child._type == 'GFPage':
        page = child
        break

    if not page:
      # Create a single page on which to place blocks and labels
      page = self.AddElement('page', self.form,
         {  'name': 'pg%s' % tableKey })


    # We will need a block to hold our entries...

    # The basic attributes for a block
    attrs = {'name' : 'blk%s' % tableKey,
             'datasource': datasource.name}

    # If this is a multirecord form, set rows at the block level.
    if multirecord:
      attrs['rows'] = 10

    # Create the block
    block = self.AddElement('block', page, attrs)

    schema = self.GetSourceSchema(self.variables['connection'],
                         self.variables['table'])


    # Create the entries and labels...


    # Make a map of all the field schemas we will need
    # We will not actually create the entries at this point
    # because we want to keep in the order that the user
    # specified
    fields = {}
    for field in schema.getChildSchema(schema):
      if field.name in self.variables['fields']:
        fields[field.name] = field

    entryQueue = []
    labelQueue = []
    largestField = 0
    largestLabel = 0

    # Now, let's make all the entries
    for name in self.variables['fields']:

      field = fields[name]
      # We will use the field name as the basis for all our
      # entry and label names.  Capitalize the first letter
      # and strip all spaces...
      if len(field.name) == 1:
        fieldKey = string.upper(field.name)
      else:
        fieldKey = string.join(string.split(string.capwords( \
                     string.replace(field.name,'_',' '))),'')

      # Create a label. If the schema specified a label for a field,
      # use that as is.  If not, use the field name, replacing any '_'
      # with spaces and tacking on a colon.
      text = hasattr(field,'label') and label or \
             string.capwords(string.replace(field.name,'_',' ')) + ':'

      labelQueue.append(self.AddElement('label', page,
                  {'x':1, 'y': 1,
                   'name': "lbl%s" % fieldKey,
                   'text': text,
                   'width': len(text)}))

      # Keep track of the greatest label width
      largestLabel = largestLabel < len(text) and len(text) or largestLabel


      # Create an entry for this field.

      attrs={'x':1, 'y': 1,
             'name': "inp%s" % fieldKey,
             'field': field.name,
             'typecast': field.datatype,
             'width': 10}

      # If we have a length for the field, use this as the max_length
      # for the entry. Also, adjust the display width if necessary.
      if hasattr(field,'length'):
        attrs['max_length'] = field.length
        if field.length < 25:
          attrs['width'] = field.length
        else:
          attrs['width'] = 25

      # Keep track of the greatest field width
      largestField = largestField < attrs['width'] and \
             attrs['width'] or largestField

      # Create the entry element
      entryQueue.append(self.AddElement('entry', block, attrs))


    # Rearrange the fields and labels to snugly fit the screen
    # based on the layout arrangement selected by the user.

    # Grid/multirecord layout
    if multirecord:

      height = 13
      width = 1
      x = 1

      for i in range(len(entryQueue)):
        self.ModifyElement(labelQueue[i],x=x,y=1)
        self.ModifyElement(entryQueue[i],x=x,y=2)

        # If label width is larger than entry width, center the entry
        if entryQueue[i].width < labelQueue[i].width:
          self.ModifyElement(entryQueue[i],x=entryQueue[i].x + \
              int((labelQueue[i].width - entryQueue[i].width)/2))


        # Calculate the starting x for the next label/entry
        dx = (entryQueue[i].width < labelQueue[i].width and \
              labelQueue[i].width or entryQueue[i].width) + 1

        # Increase the form width accordingly
        width += dx
        x = x + dx

    # Single Record layout (regardless of label location)
    else:

      # Assign the starting (x,y) values for labels (l) and fields (f)
      # as well as the delta (x,y) values.
      if leftlabels:
        # Labels to the left of the entries
        lx, ly, ldx, ldy = (1, 1, 0, 1)
        fx, fy, fdx, fdy = (largestLabel + 2, 1, 0, 1)

        # Set form width
        width = largestField + largestLabel + 3

      else:
        # Labels above the entries
        lx, ly, ldx, ldy = (1, 1, 0, 3)
        fx, fy, fdx, fdy = (1, 2, 0, 3)

        # Set the form width
        width = 2 + (largestField > largestLabel and largestField \
                               or largestLabel)

      # Rearrange
      height = fy+2-fdy
      for i in range(len(entryQueue)):
        self.ModifyElement(labelQueue[i],x=lx,y=ly)
        self.ModifyElement(entryQueue[i],x=fx,y=fy)
        ly = ly + ldy
        lx = lx + ldx
        fy = fy + fdy
        fx = fx + fdx
        height += fdy

    self.ModifyElement(self.form, width=width, height=height)

    # That's it... we're done.
    return 1



############
#
# Basic information about this template
#
TemplateInformation = {
    'Product': 'forms',
    'BaseClass' : SimpleFormTemplate,
    'Name' : 'Simple form wizard',
    'Description' : 'Creates a simple single-source entry table.',
    'Version' : '0.0.1',
    'Author' : 'The GNUe Designer Team',
    'Behavior': WIZARD
}

